/* 
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 *
 * Linking the Rast statically or dynamically with other modules is
 * making a combined work based on the Rast.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of the Rast give
 * you permission to link the Rast with independent modules that
 * communicate with the Rast solely through the the Apache Portable
 * Runtime interface, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * combined work under terms of your choice, provided that every
 * copy of the combined work is accompanied by a complete copy of
 * the source code of the Rast (the version of the Rast used to
 * produce the combined work), being distributed under the terms of
 * the GNU General Public License plus this exception.  An
 * independent module is a module which is not derived from or based
 * on the Rast.
 *
 * Note that people who make modified versions of the Rast are not
 * obligated to grant this special exception for their modified
 * versions; it is their choice whether to do so.  The GNU General
 * Public License gives permission to release a modified version
 * without this exception; this exception also makes it possible to
 * release a modified version which carries forward this exception.
 */

#ifndef RAST_DB_H
#define RAST_DB_H

/**
 * @file db.h db
 */

#include "rast/error.h"
#include "rast/rast.h"
#include "rast/result.h"

/**
 * @defgroup db db
 * @{
 */

/** Options for rast_db_create() */
typedef struct {
    /** The byte order of the database */
    rast_byte_order_e byte_order;
    /** The encoding of the database */
    const char *encoding;
    /** Whether the whole text should be preseved in the database */
    int preserve_text;
    /** The property definitions */
    rast_property_t *properties;
    /** The number of the properties */
    int num_properties;
    /** The block size of .pos files */
    rast_size_t pos_block_size;
} rast_db_create_option_t;

/**
 * Create options for rast_db_create().
 * @param pool The pool to allocate the memory out of
 * @return The created options
 */
rast_db_create_option_t *rast_db_create_option_create(apr_pool_t *pool);

/** Options for rast_db_open() */
typedef struct {
    /** The number of registered characters to start sync */
    int sync_threshold_chars;
} rast_db_open_option_t;

/**
 * Create options for rast_db_open().
 * @param pool The pool to allocate the memory out of
 * @return The created options
 */
rast_db_open_option_t *rast_db_open_option_create(apr_pool_t *pool);

#define RAST_DB_RDWR    0       /**< Opening database in read-write mode */
#define RAST_DB_RDONLY  1       /**< Opening database in read-only mode */

typedef struct rast_db_t rast_db_t;

struct rast_db_t {
    /** The pool associated with the database */
    apr_pool_t *pool;

    rast_error_t *(*db_sync)(rast_db_t *db);
    rast_error_t *(*db_close)(rast_db_t *db);
    rast_error_t *(*db_register)(rast_db_t *db,
                                 const char *text, rast_size_t nbytes,
                                 rast_value_t *properties,
                                 rast_doc_id_t *doc_id);
    rast_error_t *(*db_search)(rast_db_t *db, const char *query,
                               rast_search_option_t *options,
                               rast_result_t **result, apr_pool_t *pool);
    rast_error_t *(*db_delete)(rast_db_t *db, rast_doc_id_t doc_id);
    rast_error_t *(*db_update)(rast_db_t *db,
                               rast_doc_id_t old_doc_id,
                               const char *text, rast_size_t nbytes,
                               rast_value_t *properties,
                               rast_doc_id_t *new_doc_id);
    rast_error_t *(*db_get_text)(rast_db_t *db,
                                 rast_doc_id_t doc_id,
                                 char **s, rast_size_t *nbytes,
                                 apr_pool_t *pool);
    rast_byte_order_e (*db_byte_order)(rast_db_t *db);
    const char *(*db_encoding)(rast_db_t *base);
    const rast_property_t *(*db_properties)(rast_db_t *db,
                                            int *num_properties);
    int (*db_sync_threshold_chars)(rast_db_t *db);
};

typedef rast_error_t *rast_open_func_t(rast_db_t **db,
                                       const char *name, int flags,
                                       rast_db_open_option_t *options,
                                       apr_pool_t *pool);

/**
 * Add a function for rast_db_open.  If you want to know how to use this
 * function, please read xmlrpc_client.c :>.
 * @param open_func The function of rast_db_open
 */
void rast_add_open_function(rast_open_func_t *open_func);

/**
 * Create a database.  This function do not open the created database.
 * Use rast_db_open() to open the database.
 * @param name The name of the database
 * @param options The options to create a database.
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 * @see rast_db_create_option_create()
 */
rast_error_t *rast_db_create(const char *name,
                             rast_db_create_option_t *options,
                             apr_pool_t *pool);

/**
 * Open the specified database.
 * @param db The database opened
 * @param name The name of the database
 * @param flags How to open
 *          - RAST_RDRW
 *            - open for reading and writing
 *          - RAST_RDONLY
 *            - open for reading
 * @param options The options to open the database.
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 * @see rast_db_open_option_create()
 */
rast_error_t *rast_db_open(rast_db_t **db, const char *name, int flags,
                           rast_db_open_option_t *options, apr_pool_t *pool);

/**
 * Sync registered data to files.
 * @param db The database to sync
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_sync(rast_db_t *db);

/**
 * Close the database.
 * @param db The database to close
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_close(rast_db_t *db);

/**
 * Register a document to the database.
 * @param db The database
 * @param text The text of the document
 * @param nbytes The length of text in bytes
 * @param properties The property values of documents
 * @param doc_id If non-NULL, the registered document ID
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_register(rast_db_t *db,
                               const char *text, rast_size_t nbytes,
                               rast_value_t *properties,
                               rast_doc_id_t *doc_id);

/**
 * Search documents in the database.
 * @param db The database
 * @param query The query string
 * @param options search options
 * @param result The search result
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 * @see rast_search_option_create()
 */
rast_error_t *rast_db_search(rast_db_t *db,
                             const char *query, rast_search_option_t *options,
                             rast_result_t **result, apr_pool_t *pool);

/**
 * Delete the document from the database.
 * @param db The database
 * @param doc_id The document ID
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_delete(rast_db_t *db, rast_doc_id_t doc_id);

/**
 * Update the document in the database.
 * @param db The database
 * @param old_doc_id The document ID
 * @param text The text of the document
 * @param nbytes The length of text in bytes
 * @param properties The property values of documents
 * @param new_doc_id If non-NULL, Updated document ID
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_update(rast_db_t *db,
                             rast_doc_id_t old_doc_id,
                             const char *text, rast_size_t nbytes,
                             rast_value_t *properties,
                             rast_doc_id_t *new_doc_id);

/**
 * Get the text of the document from the database.
 * @param db The database
 * @param doc_id The document ID
 * @param s The text of the document
 * @param nbytes The length of the text in bytes
 * @param pool The pool to allocate the memory out of
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_db_get_text(rast_db_t *db, rast_doc_id_t doc_id,
                               char **s, rast_size_t *nbytes,
                               apr_pool_t *pool);

/**
 * Return the byte order of the database.
 * @param db The database
 * @return The byte order of the database
 * @see rast_local_db_byte_order()
 */
rast_byte_order_e rast_db_byte_order(rast_db_t *db);

/**
 * Return the encoding of the database.
 * @param db The database
 * @return The encoding of the database
 * @see rast_local_db_encoding()
 */
const char *rast_db_encoding(rast_db_t *db);

/**
 * Return the property definitions of the database.
 * @param db The database
 * @param num_properties The number of properties
 * @return The property definitions
 */
const rast_property_t *rast_db_properties(rast_db_t *db, int *num_properties);

/**
 * Return the number of registered characters to start sync.
 * @param db The database
 * @return The number of registered characters to start sync
 * @see rast_local_db_sync_threshold_chars()
 */
int rast_db_sync_threshold_chars(rast_db_t *db);

/** @} */

#endif
