/* 
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 *
 * Linking the Rast statically or dynamically with other modules is
 * making a combined work based on the Rast.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of the Rast give
 * you permission to link the Rast with independent modules that
 * communicate with the Rast solely through the the Apache Portable
 * Runtime interface, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * combined work under terms of your choice, provided that every
 * copy of the combined work is accompanied by a complete copy of
 * the source code of the Rast (the version of the Rast used to
 * produce the combined work), being distributed under the terms of
 * the GNU General Public License plus this exception.  An
 * independent module is a module which is not derived from or based
 * on the Rast.
 *
 * Note that people who make modified versions of the Rast are not
 * obligated to grant this special exception for their modified
 * versions; it is their choice whether to do so.  The GNU General
 * Public License gives permission to release a modified version
 * without this exception; this exception also makes it possible to
 * release a modified version which carries forward this exception.
 */

#ifndef RAST_ENCODING_H
#define RAST_ENCODING_H

/**
 * @file encoding.h encoding
 */

#include <apr_pools.h>

#include "rast/rast.h"
#include "rast/error.h"

/**
 * @defgroup encoding encoding
 * @{
 */

typedef struct rast_encoding_module_t rast_encoding_module_t;

/** A structure that represents a character */
typedef struct {
    rast_encoding_module_t *encoding_module;
    const char *ptr;
    int nbytes;
} rast_char_t;

/** A structure that represents a token */
typedef struct {
    const char *ptr;
    int nbytes;
    int nchars;
    rast_pos_t pos;
    int is_complete;
} rast_token_t;

/** A structure that represents a tokenizer */
typedef struct {
    rast_encoding_module_t *encoding_module;
    apr_pool_t *pool;
    const unsigned char *ptr;
    const unsigned char *ptr_end;
    rast_pos_t pos;
    void *context;
} rast_tokenizer_t;

/** A structure that represents an encoding module definition */
struct rast_encoding_module_t {
    /**
     * Return the length of the current character.
     * @param tokenizer The tokenizer.
     * @param len The length of the current character in bytes
     * @return RAST_OK if succeeded, error otherwise
     */
    rast_error_t *(*get_char_len)(rast_tokenizer_t *tokenizer,
                                  rast_size_t *len);

    /**
     * Return the current token.
     * @param tokenizer The tokenizer.
     * @param token The current token
     * @return RAST_OK if succeeded, error otherwise
     */
    rast_error_t *(*get_token)(rast_tokenizer_t *tokenizer,
                               rast_token_t *token);

    /**
     * Return the offset to the next token.
     * @param tokenizer The tokenizer.
     * @param byte_offset The byte offset to the next token
     * @param char_offset The character offset to the next token
     * @return RAST_OK if succeeded, error otherwise
     */
    rast_error_t *(*get_next_offset)(rast_tokenizer_t *tokenizer,
                                     rast_size_t *byte_offset,
                                     rast_size_t *char_offset);

    /**
     * Normalize text.  This function may change number of characters.
     * @param pool The pool to allocate the memory out of
     * @param src The source string
     * @param src_len The length of the source string
     * @param dst The destination string
     * @param dst_len The length of the destination string
     */
    void (*normalize_text)(apr_pool_t *pool,
                           const char *src, rast_size_t src_len,
                           char **dst, rast_size_t *dst_len);

    /**
     * Normalize each character in src.  This function should not change
     * number of characters.
     * @param pool The pool to allocate the memory out of
     * @param src The source string
     * @param src_len The length of the source string
     * @param dst The destination string
     * @param dst_len The length of the destination string
     */
    void (*normalize_chars)(apr_pool_t *pool,
                            const char *src, rast_size_t src_len,
                            char **dst, rast_size_t *dst_len);

    /**
     * Check whether a character is a space character or not.
     * @param ch The character to check
     * @return 1 if ch is a space character, 0 otherwise
     */
    int (*is_space)(rast_char_t *ch);
};

/**
 * Load encoding modules.
 * @param dirname The name of the directory where encoding modules are located.
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_load_encoding_modules(const char *dirname);

/**
 * Unload encoding modules.
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_unload_encoding_modules();

/**
 * Get an encoding module.
 * @param name The name of the encoding module
 * @param module The encoding module
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_get_encoding_module(const char *name,
                                       rast_encoding_module_t **module);

/**
 * Normalize the text by the specified encoding_module.  This function calls
 * rast_encoding_module_t::normalize_text at first, then calls
 * rast_encoding_module_t::normalize_chars.
 * @param encoding_module The encoding module for normalizing.
 * @param s The source string
 * @param nbytes The length of the source string in bytes.
 * @param new_nbytes The length of the normalized string in bytes.
 * @param pool The pool to allocate the memory out of
 * @return The normalized string
 */
char *rast_normalize_text(rast_encoding_module_t *encoding_module,
                          const char *s, rast_size_t nbytes,
                          rast_size_t *new_nbytes, apr_pool_t *pool);

rast_tokenizer_t *rast_char_tokenizer_create(apr_pool_t *pool,
                                             rast_encoding_module_t *,
                                             const char *s,
                                             rast_size_t nbytes);
rast_error_t *rast_char_tokenizer_next(rast_tokenizer_t *tokenizer);
rast_error_t *rast_char_tokenizer_get_current(rast_tokenizer_t *tokenizer,
                                              rast_char_t *ch);
int rast_char_tokenizer_is_done(rast_tokenizer_t *tokenizer);

rast_tokenizer_t *rast_register_tokenizer_create(apr_pool_t *pool,
                                                 rast_encoding_module_t *,
                                                 const char *s,
                                                 rast_size_t nbytes);
rast_error_t *rast_register_tokenizer_next(rast_tokenizer_t *tokenizer);
rast_error_t *rast_register_tokenizer_get_current(rast_tokenizer_t *tokenizer,
                                                  rast_token_t *token);
int rast_register_tokenizer_is_done(rast_tokenizer_t *tokenizer);

rast_tokenizer_t *rast_search_tokenizer_create(apr_pool_t *pool,
                                               rast_encoding_module_t *,
                                               const char *s,
                                               rast_size_t nbytes);
rast_error_t *rast_search_tokenizer_next(rast_tokenizer_t *tokenizer);
rast_error_t *rast_search_tokenizer_get_current(rast_tokenizer_t *tokenizer,
                                                rast_token_t *token);
int rast_search_tokenizer_is_done(rast_tokenizer_t *tokenizer);

int rast_count_chars(rast_encoding_module_t *encoding_module,
                     const char *s, rast_size_t nbytes, apr_pool_t *pool);
int rast_char_is_space(rast_char_t *ch);

/** @} */

#endif /* RAST_ENCODING_H */

/* vim: set filetype=c sw=4 expandtab : */
