/* 
 * Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
 *
 * This file is part of Rast.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 *
 * Linking the Rast statically or dynamically with other modules is
 * making a combined work based on the Rast.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of the Rast give
 * you permission to link the Rast with independent modules that
 * communicate with the Rast solely through the the Apache Portable
 * Runtime interface, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * combined work under terms of your choice, provided that every
 * copy of the combined work is accompanied by a complete copy of
 * the source code of the Rast (the version of the Rast used to
 * produce the combined work), being distributed under the terms of
 * the GNU General Public License plus this exception.  An
 * independent module is a module which is not derived from or based
 * on the Rast.
 *
 * Note that people who make modified versions of the Rast are not
 * obligated to grant this special exception for their modified
 * versions; it is their choice whether to do so.  The GNU General
 * Public License gives permission to release a modified version
 * without this exception; this exception also makes it possible to
 * release a modified version which carries forward this exception.
 */

#ifndef RAST_H
#define RAST_H

/**
 * @file rast.h rast
 */

#include <apr.h>

#include "rast/error.h"
#include "rast/types.h"

/**
 * @defgroup rast rast
 * @{
 */

#define RAST_RDWR       0       /**< This is OBSOLETE.  Please replace to RAST_DB_RDWR */
#define RAST_RDONLY     1       /**< This is OBSOLETE.  Please replace to RAST_DB_RDONLY */

#define RAST_BUFSIZ 1024

#define RAST_RESULT_ALL_ITEMS -1

#define RAST_CALC_SCORE_STANDALONE -1

/** Enum for sort methods */
typedef enum {
    RAST_SORT_METHOD_SCORE,      /**< Sort by score */
    RAST_SORT_METHOD_PROPERTY    /**< Sort by property */
} rast_sort_method_e;

/** Enum for sort order */
typedef enum {
    /**
     * descending if sort method is RAST_SORT_METHOD_SCORE,
     * ascending otherwise
     */
    RAST_SORT_ORDER_DEFAULT,
    RAST_SORT_ORDER_ASCENDING,   /**< ascending order */
    RAST_SORT_ORDER_DESCENDING   /**< descending order */
} rast_sort_order_e;

/** Enum for sort methods */
typedef enum {
    RAST_SCORE_METHOD_NONE,      /**< Do not calculate scores */
    RAST_SCORE_METHOD_TFIDF,     /**< Use TF-IDF method */
} rast_score_method_e;

/** Options for search */
typedef struct {
    /** The start number of requested items.  start_no starts from 0. */
    rast_doc_id_t start_no;
    /** The number of requested items */
    int num_items;
    /** Whether summary is needed or not */
    int need_summary;
    /** The number of characters of summary */
    rast_size_t summary_nchars;
    /** The sort method */
    rast_sort_method_e sort_method;
    /** The sort property */
    const char *sort_property;
    /** The sort order */
    rast_sort_order_e sort_order;
    /** The score method */
    rast_score_method_e score_method;
    /** The requested properties */
    const char **properties;
    /** The number of requested properties */
    int num_properties;
    /** The number of documents in all of search nodes */
    int all_num_docs;
    /** The number of documents that contains the term */
    int *terms;
    /** The number of terms */
    int num_terms;
} rast_search_option_t;

/** Enum for byte order */
typedef enum {
    RAST_LITTLE_ENDIAN = 0,  /**< Little endian */
    RAST_BIG_ENDIAN = 1,     /**< Big endian */
    RAST_NATIVE_ENDIAN = 2,  /**< Native endian */

    RAST_UNKNOWN_ENDIAN = -1   /**< Not implemented database */
} rast_byte_order_e;

static inline rast_byte_order_e
rast_check_byte_order()
{
    union {
        long long_value;
        char char_value[sizeof(long)];
    } data;

    data.long_value = 1;
    return data.char_value[0] == 1 ? RAST_LITTLE_ENDIAN : RAST_BIG_ENDIAN;
}

static inline apr_uint32_t
rast_swap32(apr_uint32_t x)
{
    return ((x & 0x000000FF) << 24) |
        ((x & 0x0000FF00) << 8) |
        ((x & 0x00FF0000) >> 8) |
        ((x & 0xFF000000) >> 24);
}

static inline rast_size_t
rast_fix_byte_order(rast_size_t n, int is_native)
{
    return is_native ? n : rast_swap32(n);
}

/**
 * Set the function to be called when an allocation failure occurs.
 */
void rast_set_abort_func(apr_abortfunc_t func);

/**
 * Create a pool which calls the function specified by rast_set_abort_func()
 * when an allocation failure occurs.
 * @param parent The parent pool
 * @return The created pool
 */
apr_pool_t *rast_pool_create(apr_pool_t *parent);

/**
 * Setup any Rast internal data structures.  This MUST be the first function
 * called for any Rast library.
 * @return RAST_OK if succeeded, error otherwise
 */
rast_error_t *rast_initialize();

/**
 * Tear down any Rast internal data structures.
 */
void rast_finalize();

/**
 * Get global pool.
 * @return global pool if initialized, NULL otherwise
 */
apr_pool_t *rast_get_global_pool();

/**
 * Create options for search.
 * @param pool The pool to allocate the memory out of
 * @return The created options
 */
rast_search_option_t *rast_search_option_create(apr_pool_t *pool);

/** @} */

#endif /* RAST_H */

/* vim: set filetype=c sw=4 expandtab : */
